package com.tzld.piaoquan.recommend.server.remote;

import io.milvus.client.MilvusClient;
import io.milvus.grpc.SearchResultData;
import io.milvus.grpc.SearchResults;
import io.milvus.param.ConnectParam;
import io.milvus.param.R;
import io.milvus.param.dml.SearchParam;
import io.milvus.pool.ClientPool;
import io.milvus.pool.PoolClientFactory;
import io.milvus.pool.PoolConfig;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.time.Duration;

/**
 * https://github.com/milvus-io/milvus-sdk-java/blob/2.4/examples/main/java/io/milvus/v2/ClientPoolExample.java
 *
 * @author dyp
 */
@Component
@Slf4j
public class ZillizRemoteService {

    private CustomMilvusClientPool pool;
    private String poolKey = "milvus";
    @Value("${zillizUri:}")
    private String zillizUri;
    @Value("${zillizToken:}")
    private String zillizToken;


    @PostConstruct
    public void init() {
        try {
            // milvus 使用 io.grpc.internal.DnsNameResolverProvider
            ConnectParam connectConfig = ConnectParam.newBuilder()
                    .withUri(zillizUri)
                    .withToken(zillizToken)
                    .build();
            PoolConfig poolConfig = PoolConfig.builder()
                    .maxIdlePerKey(1000) // max idle clients per key
                    .maxTotalPerKey(1000) // max total(idle + active) clients per key
                    .maxTotal(1000) // max total clients for all keys
                    .maxBlockWaitDuration(Duration.ofMillis(200)) // getClient() will wait 5 seconds if no idle
                    // client available
                    .minEvictableIdleDuration(Duration.ofSeconds(10L)) // if number of idle clients is larger than maxIdlePerKey, redundant idle clients will be evicted after 10 seconds
                    .build();
            pool = new CustomMilvusClientPool(poolConfig, connectConfig);
        } catch (Exception e) {
            log.error("MilvusClientV2Pool init error ", e);
            throw new RuntimeException();
        }
    }

    public SearchResultData search(SearchParam param) {
        MilvusClient client = pool.getClient(poolKey);
        R<SearchResults> searchRet = client.search(param);
        pool.returnClient(poolKey, client);

        if (searchRet.getStatus() != R.Status.Success.getCode()) {
            throw new RuntimeException("Failed to search! Error: " + searchRet.getMessage());
        }

        return searchRet.getData().getResults();
    }


    private class CustomMilvusClientPool extends ClientPool<ConnectParam, MilvusClient> {
        public CustomMilvusClientPool(PoolConfig poolConfig, ConnectParam connectParam) throws ClassNotFoundException, NoSuchMethodException {
            super(poolConfig, new PoolClientFactory(connectParam, CustomMilvusServiceClient.class.getName()));
        }
    }
}
